﻿// For an introduction to the Blank template, see the following documentation:
// http://go.microsoft.com/fwlink/?LinkId=232509
//
// Tutorial based on this article: http://blogs.msdn.com/b/davrous/archive/2012/09/07/tutorial-series-using-winjs-amp-winrt-to-build-a-fun-html5-camera-application-for-windows-8-2-4.aspx
(function () {
    "use strict";

    WinJS.Binding.optimizeBindingReferences = true;

    var app = WinJS.Application;
    var activation = Windows.ApplicationModel.Activation;

    // Using
    var Capture = Windows.Media.Capture;
    var Storage = Windows.Storage;

    // Globals
    var mediaCapture;
    var recording = false;
    var recordedFile;
    var livePreview;
    var pics;
    var videos;
    var flipView;
    var screensList;

    // To know if we have already computed the thumbnails for the current
    // image displayed or not
    var thumbnailsGenerated = false;
    var workInProgressElement;
    var appbar;
    var flyout;
    // Pointers to the 3 images tags contained in the flyout control
    var thumbnailBlur;
    var thumbnailBW;
    var thumbnailSepia;
    // Current image displayed in the flipview if the image template is currently used
    var currentImageToManipulate;
    var currentItem;
    // To mimic polymorphism
    var componentToUse;

    // The 3 type of filters processor: EaselJS, Web Workers and the WinRT C++ component from David Catuhe
    var easelJSFilters;
    var workersFilters;
    var deFormFilters;

    // Used to benchmark the time needed to apply an effect
    var startDate;

    app.onactivated = function (args) {
        if (args.detail.kind === activation.ActivationKind.launch) {
            if (args.detail.previousExecutionState !== activation.ApplicationExecutionState.terminated) {
                // TODO: This application has been newly launched. Initialize
                // your application here.
            } else {
                // TODO: This application has been reactivated from suspension.
                // Restore application state here.
            }
            Windows.Media.MediaControl.addEventListener("soundlevelchanged", soundLevelChangedHandler);

            var picturesLib = Storage.KnownFolders.picturesLibrary;
            picturesLib.createFolderAsync("ModernUI Fun Camera", Storage.CreationCollisionOption.openIfExists).then(function (folder) {
                pics = folder;
            });

            var videosLib = Storage.KnownFolders.videosLibrary;
            videosLib.createFolderAsync("ModernUI Fun Camera", Storage.CreationCollisionOption.openIfExists).then(function (folder) {
                videos = folder;
            });

            args.setPromise(WinJS.UI.processAll().then(init));
        }
    };

    function soundLevelChangedHandler() {
        if (Windows.Media.MediaControl.soundLevel === Windows.Media.SoundLevel.muted) {
            console.log("Camera is being cut because of a switching action");
        }
        else {
            startCamera();
            console.log("Restarting the camera for live preview");
        }
    }

    function capture() {
        if (document.getElementById("still").checked) {
            takePhoto();
        } else if (!recording) {
            recording = true;
            startRecording();
        } else {
            recording = false;
            stopRecording();
        }
    }

    function init() {
        screensList = new WinJS.Binding.List();
        screensList.push({ "type": "application" });
        flipView = document.getElementById("flip-view").winControl;

        WinJS.UI.setOptions(flipView, {
            itemDataSource: screensList.dataSource,
            itemTemplate: WinJS.Utilities.markSupportedForProcessing(flipViewItemTemplateRendererFunction)
        });

        // Accessing to the AppBar WinJS Control and
        // setting it as disabled by default
        appbar = document.getElementById("appbar").winControl;
        appbar.disabled = true;

        // Everytime we're navigating into the flip view
        flipView.addEventListener("pageselected", function () {
            currentItem = screensList.getAt(flipView.currentPage);
            // making the application bar available only if an image
            // is displayed in the flipview
            if (currentItem.type === "image") {
                appbar.disabled = false;
            }
            else {
                appbar.disabled = true;
            }

            // Resetting our boolean to false when the user
            // navigate to another image to indicate we need
            // to rebuild the thumbnails
            thumbnailsGenerated = false;
        });

        flyout = document.getElementById("filtersFlyout");
        // Getting the 3 images available in the flyout and adding the applyEffect handler
        // on the click event
        thumbnailBlur = flyout.querySelector("#effectBlur");
        thumbnailBlur.addEventListener("click", function () {
            applyEffect("Blur");
        }, false);

        thumbnailBW = flyout.querySelector("#effectBW");
        thumbnailBW.addEventListener("click", function () {
            applyEffect("BW");
        }, false);

        thumbnailSepia = flyout.querySelector("#effectSepia");
        thumbnailSepia.addEventListener("click", function () {
            applyEffect("Sepia");
        }, false);

        // We're using the very same flyout for each case but we then just need
        // to indicate later on which component to use for the filter processing
        document.getElementById("cmdEaselJSFilters").addEventListener("click", function () {
            componentToUse = "EaselJS";
            displayFlyout()
        }, false);
        document.getElementById("cmdWorkersFilters").addEventListener("click", function () {
            componentToUse = "Workers";
            displayFlyout();
        }, false);
        document.getElementById("cmdDeFormFilters").addEventListener("click", function () {
            componentToUse = "DeForm";
            displayFlyout();
        }, false);

        // Displayed while the filter is being applied to the image
        workInProgressElement = document.getElementsByClassName("workInProgress")[0];
    }

    function displayFlyout() {
        // Generating the effects in thumbnails
        // to let the user preview them before applying
        // The thumbnailsGenerated boolean is handling our cache
        if (!thumbnailsGenerated) {
            currentImageToManipulate = new Image();
            currentImageToManipulate.onload = function () {
                // updateFlipViewItem is the callback method called when a filter has been applied
                easelJSFilters = new EaselJSFilters.Filters(currentImageToManipulate, updateFlipViewItem);
                // the last parameter indicate the number of workers you'd like to use. 4 here. 
                workersFilters = new WorkersFilters.Filters(currentImageToManipulate, updateFlipViewItem, 4);
                // We're always using EaselJS to generate the preview thumbnails
                var imagesArray = [thumbnailBlur, thumbnailBW, thumbnailSepia];
                easelJSFilters.generateThumbnails(imagesArray);
                // We need to generate thumbnails here
                thumbnailsGenerated = true;
            };
            currentImageToManipulate.src = currentItem.url;
        }

        // displaying directly the flyout if the thumbnails have
        // already been generated
        flyout.winControl.show(document.getElementById("cmd" + componentToUse + "Filters"));
    }

    function applyEffect(type) {
        var currentComponent;
        WinJS.Utilities.removeClass(workInProgressElement, "hidden");
        flyout.winControl.hide();

        WinJS.Promise.timeout(100).then(function () {
            startDate = new Date();

            switch (componentToUse) {
                case "EaselJS":
                    currentComponent = easelJSFilters;
                    break;
                case "Workers":
                    currentComponent = workersFilters;
                    break;
                case "DeForm":
                    currentComponent = deFormFilters;
                    break;
            }

            switch (type) {
                case "Blur":
                    currentComponent.applyBlurEffect();
                    break;
                case "BW":
                    currentComponent.applyBWEffect();
                    break;
                case "Sepia":
                    currentComponent.applySepiaEffect();
                    break;
            }
        });

        thumbnailsGenerated = false;
    }

    function updateFlipViewItem(url) {
        var diff = new Date() - startDate;
        console.log("Process done in " + diff);
        var messageDialog = new Windows.UI.Popups.MessageDialog("Process done in " + diff + " ms.");
        messageDialog.showAsync();

        currentItem.url = url;
        screensList.setAt(flipView.currentPage, currentItem);
        WinJS.Utilities.addClass(workInProgressElement, "hidden");
    }

    function flipViewItemTemplateRendererFunction(itemPromise) {
        return itemPromise.then(function (currentItem) {
            var templateSelector = "#" + currentItem.data.type + "Template";
            var controlTemplate = document.querySelector(templateSelector).winControl;
            var template = controlTemplate.renderItem(itemPromise);
            template.renderComplete = template.renderComplete.then(function (elem) {
                switch (currentItem.data.type) {
                    case "application":
                        releaseMediaCapture();
                        livePreview = elem.querySelector("#live-preview");
                        livePreview.addEventListener("click", capture);
                        console.log("starting the camera");
                        startCamera();
                        break;
                    case "video":
                        addPlayControls(elem);
                        break;
                }
            });
            return template.element;
        })
    }

    function addPlayControls(elements) {
        var button = elements.querySelector("button");
        var video = elements.querySelector(".media video");

        function _play() {
            video.play();
            WinJS.UI.Animation.fadeOut(button);
        }

        video.addEventListener("click", _play);
        button.addEventListener("click", _play);

        video.addEventListener("ended", function () {
            WinJS.UI.Animation.fadeIn(button);
        });
    }

    function startCamera() {
        mediaCapture = new Capture.MediaCapture();
        mediaCapture.initializeAsync().then(function () {
            livePreview.src = URL.createObjectURL(mediaCapture);
            livePreview.play();
        });

        livePreview.addEventListener("pause", handlePause);
    }

    function handlePause() {
        console.log("pause invoked...");
        livePreview.play();
    }

    function releaseMediaCapture() {
        if (livePreview) {
            livePreview.src = null;
            livePreview = null;
        }
        mediaCapture = null;
        console.log("releasemediacapture");
    }

    function startRecording() {
        document.getElementById("capture-status").innerText = "Recording.  Tap screen to stop."
        videos.createFileAsync("video.mp4", Windows.Storage.CreationCollisionOption.generateUniqueName).then(function (file) {
            var profile = Windows.Media.MediaProperties.MediaEncodingProfile.createMp4(Windows.Media.MediaProperties.VideoEncodingQuality.auto);

            mediaCapture.startRecordToStorageFileAsync(profile, file).then(function () {
                recordedFile = file;
            });
        });
    }

    function stopRecording() {
        mediaCapture.stopRecordAsync().then(function () {
            document.getElementById("capture-status").innerText = "Tap screen to capture media";
            console.log("Video saved on disk");
            var videoUrl = URL.createObjectURL(recordedFile, { oneTimeOnly: false });
            recordedFile.getThumbnailAsync(Windows.Storage.FileProperties.ThumbnailMode.singleItem).then(function (thumbnail) {
                var thumbnailUrl = URL.createObjectURL(thumbnail, { oneTimeOnly: false });
                addItem(videoUrl, "video", thumbnailUrl);
            });
        });
    }

    function takePhoto() {
        pics.createFileAsync("photo.jpg", Storage.CreationCollisionOption.generateUniqueName).then(function (file) {
            var photoProperties = Windows.Media.MediaProperties.ImageEncodingProperties.createJpeg();

            mediaCapture.capturePhotoToStorageFileAsync(photoProperties, file).then(function () {
                console.log("Image saved on disk.");
                var thumbnailUrl = URL.createObjectURL(file, { oneTimeOnly: false });
                addItem(thumbnailUrl, "image", thumbnailUrl);
            });
        });
    }

    function previewSlide(thumbnail) {
        document.getElementById("slide").winControl.render({ "url": thumbnail }).then(function (elements) {
            var slide = elements.children[0];
            var handler = slide.addEventListener("MSAnimationEnd", function _removeSlide(e) {
                slide.removeEventListener("MSAnimationEnd", handler);
                document.body.removeChild(slide);
            });
            document.body.appendChild(slide);
        });
    }

    function addItem(file, type, thumbnail) {
        screensList.unshift({ "url": file, "type": type });
        previewSlide(thumbnail);
    }

    app.oncheckpoint = function (args) {
        // TODO: This application is about to be suspended. Save any state
        // that needs to persist across suspensions here. You might use the
        // WinJS.Application.sessionState object, which is automatically
        // saved and restored across suspension. If you need to complete an
        // asynchronous operation before your application is suspended, call
        // args.setPromise().
    };

    app.start();
})();
