﻿(function () {
    "use strict";

    var Filters = WinJS.Class.define(
        // The constructor needs an HTML Image Element
        // and a function that will be callbacked at the
        // end of the filter processing 
        function (image, callback) {
            this._currentImageToManipulate = image;
            this._callbackFunction = callback;
            // Creating the full resolution Bitmap object and a 20% one to generate quickly
            // some thumbnails if requested
            this._bmpFull = new createjs.Bitmap(this._currentImageToManipulate);
            this._bmpFull.cache(0, 0, this._currentImageToManipulate.width, this._currentImageToManipulate.height);
            this._bmpThumb = new createjs.Bitmap(this._currentImageToManipulate);
            this._bmpThumb.cache(0, 0, this._currentImageToManipulate.width, this._currentImageToManipulate.height, 0.2);
        },
        {
            _applyFilter: function (bmp) {
                // the filter is really applied after this line of code
                // this is not an asynchronous call and it could take
                // a lot of time to compute. It could then block the UI Thread
                bmp.updateCache();
                return bmp.getCacheDataURL();
            },
            _applyBlurEffect: function (bmp) {
                var blurFilter = new createjs.BoxBlurFilter(10, 2, 2);
                var margins = blurFilter.getBounds();
                bmp.filters = [blurFilter];
                return this._applyFilter(bmp);
            },
            _applyBWEffect: function (bmp) {
                var greyScaleFilter = new createjs.ColorMatrixFilter([
                    0.33, 0.33, 0.33, 0, 0, // red
                    0.33, 0.33, 0.33, 0, 0, // green
                    0.33, 0.33, 0.33, 0, 0, // blue
                    0, 0, 0, 1, 0  // alpha
                ]);

                bmp.filters = [greyScaleFilter];
                return this._applyFilter(bmp);
            },
            _applySepiaEffect: function (bmp) {
                var sepiaFilter = new createjs.ColorMatrixFilter([
                    0.393, 0.769, 0.189, 0, 0, // red
                    0.349, 0.686, 0.168, 0, 0, // green
                    0.272, 0.534, 0.131, 0, 0, // blue
                    0, 0, 0, 1, 0  // alpha
                ]);

                bmp.filters = [sepiaFilter];
                return this._applyFilter(bmp);
            },
            generateThumbnails: function (imagesArray) {
                imagesArray[0].src = this._applyBlurEffect(this._bmpThumb);
                imagesArray[1].src = this._applyBWEffect(this._bmpThumb);
                imagesArray[2].src = this._applySepiaEffect(this._bmpThumb);
            },
            applyBlurEffect: function () {
                var fullImageModifiedUrl = this._applyBlurEffect(this._bmpFull);
                this._callbackFunction(fullImageModifiedUrl);
            },
            applyBWEffect: function () {
                var fullImageModifiedUrl = this._applyBWEffect(this._bmpFull);
                this._callbackFunction(fullImageModifiedUrl);
            },
            applySepiaEffect: function () {
                var fullImageModifiedUrl = this._applySepiaEffect(this._bmpFull);
                this._callbackFunction(fullImageModifiedUrl);
            }
        }
    );

    WinJS.Namespace.define("EaselJSFilters", { Filters: Filters });
}());