﻿(function () {
    "use strict";

    var Filters = WinJS.Class.define(
    // The constructor needs an HTML Image Element
    // and a function that will be callbacked at the
    // end of the filter processing and the number of
    // workers we'd like to launch
        function (image, callback, workersCount) {
            this._currentImageToManipulate = image;
            this._callbackFunction = callback;
            this._workersCount = workersCount;
            // Creating a working canvas to draw the image and
            // access to every pixel of it
            this._canvas = document.createElement("canvas");
            this._canvas.width = this._currentImageToManipulate.width;
            this._canvas.height = this._currentImageToManipulate.height;
            this._tempContext = this._canvas.getContext("2d");

            // Drawing the source image into the target canvas
            this._tempContext.drawImage(this._currentImageToManipulate, 0, 0, this._currentImageToManipulate.width, this._currentImageToManipulate.height);
        },
        {
            _applyFilter: function (type) {
                var len = this._canvas.width * this._canvas.height * 4; // 4 = RGBA                
                // This is the length of array sent to the worker 
                var segmentLength = len / this._workersCount;
                // Height of the picture chunck for every worker                  
                this._blockSize = this._canvas.height / this._workersCount; this._finished = 0;

                // Launching every worker
                for (var index = 0; index < this._workersCount; index++) {
                    var worker = new Worker("/js/workersFilters/pictureProcessor.js");
                    var that = this;

                    // we need a little closure for the post back message
                    worker.onmessage = function (e) {
                        that._onWorkEnded(e, that);
                    };

                    // Getting the picture
                    var canvasData = this._tempContext.getImageData(0, this._blockSize * index, this._canvas.width, this._blockSize);

                    // Sending canvas data to the worker using a copy memory operation
                    worker.postMessage({ data: canvasData, index: index, length: segmentLength, type: type });
                }
            },
            applyBlurEffect: function () {
                this._applyFilter("Blur");
            },
            applyBWEffect: function () {
                this._applyFilter("BW");
            },
            applySepiaEffect: function () {
                this._applyFilter("Sepia");
            },
            // Function called when a job is finished
            _onWorkEnded: function (e, that) {
                // Data is retrieved using a memory clone operation
                var canvasData = e.data.result;
                var index = e.data.index;

                // Copying back canvas data to canvas
                that._tempContext.putImageData(canvasData, 0, that._blockSize * index);

                that._finished++;

                if (that._finished == that._workersCount) {
                    that._callbackFunction(that._canvas.toDataURL());
                }
            }
        }
    );

    WinJS.Namespace.define("WorkersFilters", { Filters: Filters });
}());