﻿(function () {
    "use strict";

    var Filters = WinJS.Class.define(
    // The constructor needs an HTML Image Element
    // and a function that will be callbacked at the
    // end of the filter processing 
        function (image, callback) {
            this._currentImageToManipulate = image;
            this._callbackFunction = callback;

            // We're using again a working canvas
            // But this time it's in order to have access to the memory stream
            // attached to the canvas blob element
            var canvas = document.createElement("canvas");
            canvas.width = this._currentImageToManipulate.width;
            canvas.height = this._currentImageToManipulate.height;
            var tempContext = canvas.getContext("2d");

            // Drawing the source image into the target canvas
            tempContext.drawImage(this._currentImageToManipulate, 0, 0, this._currentImageToManipulate.width, this._currentImageToManipulate.height);
            var blob = canvas.msToBlob();
            // The WinRT component will only understand streams and doesn't know about the HTML5 canvas element
            this._stream = blob.msDetachStream();
        },
        {
            _applyEffect: function (effects) {
                var imageManipulator = new DeForm.ImageManipulator(this._stream);
                for (var i = 0; i < effects.length; i++) {
                    imageManipulator.addEffectDescription(effects[i]);
                }
                // The filters are really applied after this line of code
                imageManipulator.manipulate(false);

                // Now we need to transform the stream manipulated by the C++ component into something 
                // useable in HTML5. First thing is to use WinRT to create an InMemoryRandonAccessStream
                var inMemoryRandomAccessStream = new Windows.Storage.Streams.InMemoryRandomAccessStream();
                // Then we're saving the stream sent back by the C++ component into it
                imageManipulator.saveToStream(inMemoryRandomAccessStream, DeForm.CompressionMode.png);
                // From this memory stream, we now need to have an object useable in HTML.
                // The blob is perfect for that
                var blob = window.MSApp.createBlobFromRandomAccessStream("image/png", inMemoryRandomAccessStream);
                // createObjectURL supports the blob type
                var url = URL.createObjectURL(blob);
                this._callbackFunction(url);
            },
            applyBlurEffect: function () {
                var blurEffect = new DeForm.GaussianBlurEffectDescription(5.0, DeForm.GaussianBlurOptimization.Quality, DeForm.GaussianBlurBorderMode.Hard);
                this._applyEffect([blurEffect]);
            },
            applyBWEffect: function () {
                var BWEffect = new DeForm.ColorMatrixEffectDescription();
                BWEffect.colorMatrix = [0.2125, 0.2125, 0.2125, 0.0,
                    0.7154, 0.7154, 0.7154, 0.0,
                    0.0721, 0.0721, 0.0721, 0.0,
                    0.0, 0.0, 0.0, 1.0,
                    0.0, 0.0, 0.0, 0.0];

                this._applyEffect([BWEffect]);
            },
            applySepiaEffect: function () {
                var BWEffect = new DeForm.ColorMatrixEffectDescription();
                BWEffect.colorMatrix = [0.2125, 0.2125, 0.2125, 0.0,
                    0.7154, 0.7154, 0.7154, 0.0,
                    0.0721, 0.0721, 0.0721, 0.0,
                    0.0, 0.0, 0.0, 1.0,
                    0.0, 0.0, 0.0, 0.0];

                var SepiaEffect = new DeForm.ColorMatrixEffectDescription();
                SepiaEffect.colorMatrix = [0.90, 0.0, 0.0, 0.0,
                    0.0, 0.70, 0.0, 0.0,
                    0.0, 0.0, 0.30, 0.0,
                    0.0, 0.0, 0.0, 1.0,
                    0.0, 0.0, 0.0, 0.0];

                var effects = [BWEffect, SepiaEffect];

                this._applyEffect(effects);
            }
        }
    );

    WinJS.Namespace.define("DeFormFilters", { Filters: Filters });
}());